<?php

/*
 * Created on 01.02.2021
 * HomePage: https://xentr.net
 * Copyright (c) 2019-2021 XENTR | XenForo Add-ons - Styles -  All Rights Reserved
 */

namespace XENTR\Stories\Admin\Controller;

use XF\Mvc\ParameterBag;
use \XF\Admin\Controller\AbstractController;

class Story extends AbstractController
{

    public function actionIndex(){
        $page = $this->filterPage();
        $perPage = \XF::options()->xtrStoryACPList;
		
		$finder = \XF::finder('XENTR\Stories:Story');
		$finder->with('User');
		$finder->order('story_create_date', 'DESC');
		$finder->limitByPage($page, $perPage);
		
        $stories = $finder->fetch();
        $total = $finder->total();
		
        $viewParams = [
			'totalStory' => $total,
            'stories' => $stories,
            'page' => $page,
            'perPage' => $perPage
        ];
		
        return $this->view('XENTR\Story:Index', 'xentr_member_stories_list', $viewParams);
    }
	
	public function actionDelete(ParameterBag $params){		
		$story = $this->assertStoryExists($params->story_id);
		
		if ($this->isPost())
		{			
			if($story->story_deleted === true){
				return $this->error(\XF::phrase('xtr_story_has_already_deleted'));
			}
			
			$story->story_deleted = true;
			$story->save();
			
			return $this->redirect($this->buildLink('story'), \XF::phrase('xtr_story_has_been_deleted'), true);
		}
		else{			
			if($story->story_deleted === true){
				return $this->error(\XF::phrase('xtr_story_has_already_deleted'));
			}
			
			$viewParams = [
				'story' => $story
			];
			
			return $this->view('XENTR\Story:Delete', 'xentr_member_story_delete', $viewParams);
		}
	}
	
	public function actionRecover(ParameterBag $params){		
		$story = $this->assertStoryExists($params->story_id);
		
		if ($this->isPost())
		{						
			$story->story_deleted = false;
			$story->story_create_date = \XF::$time;
			$story->save();
			
			return $this->redirect($this->buildLink('story'), '', true);
		}
		else{
			$view_time_opt = \XF::options()->xtrStoryMaxAppear * 3600;
			$view_time = $story->story_create_date + $view_time_opt;
			if($view_time > \XF::$time && $story->story_deleted === false){
				return $this->error(\XF::phrase('xtr_story_recover_has_already_active'));
			}
			
			$viewParams = [
				'story' => $story
			];
			
			return $this->view('XENTR\Story:Delete', 'xentr_member_story_recover', $viewParams);
		}
	}
	
	public function actionSticky(ParameterBag $params){		
		$story = $this->assertStoryExists($params->story_id);
		
		if ($this->isPost())
		{			
			if($story->story_sticky === true){
				return $this->error(\XF::phrase('xtr_story_already_sticky'));
			}
			
			$story->story_sticky = true;
			$story->save();
			
			return $this->redirect($this->buildLink('story'), '', true);
		}
		else{			
			if($story->story_sticky === true){
				return $this->error(\XF::phrase('xtr_story_already_sticky'));
			}
			
			$viewParams = [
				'story' => $story
			];
			
			return $this->view('XENTR\Story:Delete', 'xentr_member_story_sticky', $viewParams);
		}
	}
	
	public function actionNonsticky(ParameterBag $params){		
		$story = $this->assertStoryExists($params->story_id);
		
		if ($this->isPost())
		{			
			if($story->story_sticky === false){
				return $this->error(\XF::phrase('xtr_story_already_non_sticky'));
			}
			
			$story->story_sticky = false;
			$story->save();
			
			return $this->redirect($this->buildLink('story'), '', true);
		}
		else{			
			if($story->story_sticky === false){
				return $this->error(\XF::phrase('xtr_story_already_non_sticky'));
			}
			
			$viewParams = [
				'story' => $story
			];
			
			return $this->view('XENTR\Story:Delete', 'xentr_member_story_nonsticky', $viewParams);
		}
	}
	
	public function actionHardDelete(ParameterBag $params){
		$story = $this->assertStoryExists($params->story_id);
		
        $plugin = $this->plugin('XF:Delete');

        return $plugin->actionDelete(
            $story,
            $this->buildLink('story/hard-delete', $story),
			$this->buildLink('story/view', $story),
            $this->buildLink('story'),
            $story->story_message
        );
	}
	
	public function actionView(ParameterBag $params){
		$story = $this->assertStoryExists($params->story_id);
		
		$viewParams = [
			'story' => $story
		];
		
		return $this->view('XENTR\Story:View', 'xentr_member_story_view', $viewParams);
	}

    /**
     * @param string $id
     * @param array|string|null $with
     * @param null|string $phraseKey
     *
     * @return \XENTR\Stories\Entity\Story
     */
    protected function assertStoryExists($id, $with = null, $phraseKey = null){
        return $this->assertRecordExists('XENTR\Stories:Story', $id, $with, $phraseKey);
    }

}