<?php

/*
 * Created on 01.02.2021
 * HomePage: https://xentr.net
 * Copyright (c) 2019-2021 XENTR | XenForo Add-ons - Styles -  All Rights Reserved
 */

namespace XENTR\Stories\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property int|null story_id
 * @property string story_message
 * @property int user_id
 * @property int story_deleted
 * @property int story_create_date
 * @property string story_attachment_extension
 *
 * @property \XF\Entity\User User
 */
class Story extends Entity
{

    public function getAttachment(){
        $app = $this->app();
        if ($this->story_attachment_extension)
        {
            return $app->applyExternalDataUrl(
                "stories/{$this->story_id}.{$this->story_attachment_extension}"
            );
        }
        else
        {
            return null;
        }
    }

    public function isVideo(): bool
    {
		if(empty($this->story_external_media)){
			$extension = strtolower($this->story_attachment_extension);
			return \XF\Util\File::isVideoInlineDisplaySafe($extension);
		}else{
			$ext = pathinfo($this->story_external_media, PATHINFO_EXTENSION);
			return \XF\Util\File::isVideoInlineDisplaySafe($ext);
		}
    }

	protected function _preDelete() {
        /** @var \XENTR\Stories\Service\StoryAttachment $storyAttachmentService */
        $storyAttachmentService = \XF::service('XENTR\Stories:StoryAttachment', $this);
        $storyAttachmentService->deleteStory();
	}

    public static function getStructure(Structure $structure){
        $structure->table = 'xf_xtr_story';
        $structure->shortName = 'XENTR\Stories:Story';
        $structure->primaryKey = 'story_id';
        $structure->columns = [
            'story_id' => ['type' => self::UINT, 'autoIncrement' => true],
            'story_message' => ['type' => self::STR, 'required' => true],
            'story_message_url' => ['type' => self::STR, 'default' => ''],
			'story_external_media' => ['type' => self::STR, 'default' => ''],
			'story_sticky' => ['type' => self::BOOL, 'default' => false],
            'user_id' => ['type' => self::UINT, 'default' => 0],
            'story_deleted' => ['type' => self::BOOL, 'default' => false],
            'story_create_date' => ['type' => self::UINT, 'default' => \XF::$time],
            'story_attachment_extension' => ['type' => self::STR, 'default' => '']
        ];
        $structure->getters = [
            'is_video' => ['getter' => 'isVideo', 'cache' => true],
            'attachment' => ['getter' => 'getAttachment', 'cache' => true],
        ];
        $structure->relations = [
            'User' => [
                'entity' => 'XF:User',
                'type' => self::TO_ONE,
                'conditions' => 'user_id',
                'primary' => true,
            ],
        ];

        return $structure;
    }

	

}