<?php

/*
 * Created on 01.02.2021
 * HomePage: https://xentr.net
 * Copyright (c) 2019-2021 XENTR | XenForo Add-ons - Styles -  All Rights Reserved
 */

namespace XENTR\Stories\Pub\Controller;

use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;
use XF\Pub\Controller\AbstractController;

class Story extends AbstractController
{

    public function actionIndex(){
		return $this->redirect($this->buildLink('index'));
    }

    public function actionNew(){
        $visitor = \XF::visitor();
		
		if(!$visitor->hasPermission('xtrMemberStory', 'canAdd')){
			return $this->error(\XF::phrase('xtr_story_you_do_not_have_perm_add_story'));
		}
		
		/** @var \XF\Repository\Attachment $attachmentRepo */
		$attachmentRepo = $this->repository('XF:Attachment');
		
        $dayNow = date("j", \XF::$time);
        $monthNow = date("n", \XF::$time);
        $startDate = gmmktime(0,0,0, $monthNow, $dayNow);
		
		$finder = \XF::finder('XENTR\Stories:Story');
		$finder->where('user_id', $visitor->user_id);
		$finder->where('story_create_date', '>=', $startDate);
		$finder->where('story_deleted', false);
        $lastAddedTotal = $finder->total();
		
		$daily_limit = $visitor->hasPermission('xtrMemberStory', 'dailyLimit');
		
		if($daily_limit > 0 && $lastAddedTotal >= $daily_limit){
			return $this->error(\XF::phrase('xtr_story_daily_share_limit_message', [
				'count' => $daily_limit]));
		}
		
        /** @var \XENTR\Stories\Entity\Story $story */
        $story = $this->em()->create('XENTR\Stories:Story');

		if ($visitor->canUploadAndManageAttachmentsOnProfile())
		{
			$attachmentData = $attachmentRepo->getEditorData('profile_post', $visitor);
		}
		else
		{
			$attachmentData = null;
		}

        $viewParams = [
            'story' => $story,
			'attachmentData' => $attachmentData
        ];

        return $this->view('XENTR\Story:New', 'xentr_stories_add_story', $viewParams);
    }

    public function actionSave(){
        $this->assertPostOnly();

        /** @var \XENTR\Stories\Entity\Story $story */
        $story = $this->em()->create('XENTR\Stories:Story');

        $saveProcess = $this->storySaveProcess($story);
        $saveProcess->run();

        return $this->redirect($this->buildLink('index'));
    }

    protected function storySaveProcess(\XENTR\Stories\Entity\Story $story)
    {
        $visitor = \XF::visitor();
		
        $form = $this->formAction();
		
        $input = $this->filter([
            'story_message' => 'str',
            'story_message_url' => 'str',
			'story_external_media' => 'str',
        ]);
		

        $story->user_id = $visitor->user_id;
        $form->basicEntitySave($story, $input);

		if(empty($input['story_external_media'])){
			/** @var \XENTR\Stories\Service\StoryAttachment $storyAttachmentService */
			$storyAttachmentService = $this->service('XENTR\Stories:StoryAttachment', $story);
			$upload = $this->request->getFile('story_attachment', false, false);
			if ($upload)
			{
				if (!$storyAttachmentService->setStoryFromUpload($upload))
				{
					$form->logError($storyAttachmentService->getError(), 'story_attachment');
				}
				else
				{
					$form->apply(function() use ($storyAttachmentService)
					{
						$storyAttachmentService->updateStory();
					});
				}
			}else{
				$form->logError(\XF::phrase('xtr_you_need_to_select_attachment'), 'story_attachment');
			}
		}
		
		/** @var \XF\Validator\Url $urlValidator */
		$urlValidator = \XF::app()->validator('Url');
		
		$storyUser = $this->assertViewableUser($visitor->user_id);
		$storyProfileMessage = $this->plugin('XF:Editor')->fromInput('message');
		
        $form->validate(function(FormAction $form) use ($urlValidator, $story, $input)
        {			
			/* XF::options */
			$options = \XF::options();
			
			/* Control the title type */
			$msTitleType = $options->xtrStoryTitleControlType;
			
			if($msTitleType == 'regex'){
			
				if(!preg_match('/^[a-zA-Z0-9_ ,ç,Ç,ğ,Ğ,ı,İ,ö,Ö,ş,Ş,ü,Ü]*$/i', $input['story_message'])){
					$form->logError(\XF::phrase('xtr_story_title_a_z_0_9_and_underscore_only'), 'story_message');
				}
			}
			else{
				/* Check to disallow */
				$msTitleDisallowChar = $options->xtrStoryDisallowedChar;
				if (empty($msTitleDisallowChar)) {
					return;
				}
				
				/* Control the disallow word or characters */
				foreach($msTitleDisallowChar as $msTitleDisallowChar){
					
					$msTitleDisallowChar = preg_split("/(\n|\r\n)/", $msTitleDisallowChar, -1, PREG_SPLIT_NO_EMPTY);
					$msTitleDisallowChar = array_map('trim', $msTitleDisallowChar);
					$msTitleDisallowChar = array_map('preg_quote', $msTitleDisallowChar);
					$msTitleDisallowChar = implode('|', $msTitleDisallowChar);
					$msTitleDisallowChar = str_replace('#', '\#', $msTitleDisallowChar);
					
					if (preg_match('#(' . $msTitleDisallowChar . ')#i', $input['story_message'])) {
						$form->logError(\XF::phrase('xtr_story_title_control_title_must_not_contain_disallow', [
						'disallow' => $msTitleDisallowChar]), 'story_message');
					break;
					}
				}
			}
			
			/* Control the story message url */
			if(!empty($input['story_message_url'])){
				$url = $input['story_message_url'];
				$urlValidator->coerceValue($url);
				if (!$urlValidator->isValid($url))
				{
					$form->logError(\XF::phrase('please_enter_valid_url'), 'story_message_url');
				}
			}
			
			/* Control the story external media */
			if(!empty($input['story_external_media'])){
				$url = $input['story_external_media'];
				$urlValidator->coerceValue($url);
				if (!$urlValidator->isValid($url))
				{
					$form->logError(\XF::phrase('please_enter_valid_url'), 'story_external_media');
				}
				$checkLinkRepo = \XF::app()->repository('XENTR\Stories:CheckLinks');
				$allowed_url = $checkLinkRepo->checkUrl($url);
				if(!$allowed_url){
					$form->logError(\XF::phrase('xtr_story_whitelist_urls',
												['story_media_urls' => implode (', ', \XF::options()->xtrStoryWhiteList)]), 'story_external_media');
				}
			}
		});

		$form->complete(function() use ($storyUser, $storyProfileMessage, $input)
		{
			if(\XF::options()->xtrStoryProfilePost AND $storyUser->canPostOnProfile() AND !empty($storyProfileMessage))
			{
				$creator = $this->setupProfilePostCreate($storyUser->Profile, $input['story_message'], $storyProfileMessage);
				$creator->checkForSpam();
				if (!$creator->validate($errors))
				{
					return $this->error($errors);
				}
				$this->assertNotFlooding('post');
				$profilePost = $creator->save();
				
				$this->finalizeProfilePostCreate($creator);
			}
		});

        return $form;
    }
	
	/**
	 * @param \XF\Entity\ProfilePost $profilePost
	 *
	 * @return \XF\Service\ProfilePost\Creator
	 */
	protected function setupProfilePostCreate(\XF\Entity\UserProfile $userProfile, $titleMessage, $storyProfileMessage)
	{		
		/** @var \XF\Service\ProfilePost\Creator $creator */
		$creator = $this->service('XF:ProfilePost\Creator', $userProfile);
		$creator->setContent($titleMessage . ': ' . $storyProfileMessage);

		$profilePost = $creator->getProfilePost();

		if ($profilePost->ProfileUser->canUploadAndManageAttachmentsOnProfile())
		{
			$creator->setAttachmentHash($this->filter('attachment_hash', 'str'));
		}

		return $creator;
	}
	
	protected function finalizeProfilePostCreate(\XF\Service\ProfilePost\Creator $creator)
	{
		$creator->sendNotifications();

		$profilePost = $creator->getProfilePost();

		if (\XF::visitor()->user_id)
		{
			if ($profilePost->message_state == 'moderated')
			{
				$this->session()->setHasContentPendingApproval();
			}
		}
	}
	
	public function actionView(ParameterBag $params){		
		$visitor = \XF::visitor();
		
		$story = $this->assertStoryExists($params->story_id);
		
		if($story->user_id !== $visitor->user_id){
			return $this->error(\XF::phrase('xtr_story_you_do_not_have_perm_manage_story'));
		}
		
		$viewParams = [
			'story' => $story
		];
		
		return $this->view('XENTR\Story:Delete', 'xentr_member_story_view', $viewParams);
	}
	
	public function actionDelete(ParameterBag $params){
		$visitor = \XF::visitor();
		
		if(!$visitor->hasPermission('xtrMemberStory', 'canDeleteOwn')){
			return $this->error(\XF::phrase('xtr_story_you_do_not_have_perm_delete_story'));
		}
		
		$story = $this->assertStoryExists($params->story_id);
		
		if ($this->isPost())
		{
			if($story->user_id !== $visitor->user_id){
				return $this->error(\XF::phrase('xtr_story_you_do_not_have_perm_manage_story'));
			}
			
			if($story->story_deleted === true){
				return $this->error(\XF::phrase('xtr_story_has_already_deleted'));
			}
			
			$story->story_deleted = true;
			$story->save();
			
			return $this->redirect($this->buildLink('account/stories'), \XF::phrase('xtr_story_has_been_deleted'), true);
		}
		else{			
			if($story->user_id !== $visitor->user_id){
				return $this->error(\XF::phrase('xtr_story_you_do_not_have_perm_manage_story'));
			}
			
			if($story->story_deleted === true){
				return $this->error(\XF::phrase('xtr_story_has_already_deleted'));
			}
			
			$viewParams = [
				'story' => $story
			];
			
			return $this->view('XENTR\Story:Delete', 'xentr_member_story_delete', $viewParams);
		}
	}
	
	public function actionRecover(ParameterBag $params){		
		$visitor = \XF::visitor();
		
		if(!$visitor->hasPermission('xtrMemberStory', 'canRecover')){
			return $this->error(\XF::phrase('xtr_story_recover_perm_warning'));
		}
	
		$story = $this->assertStoryExists($params->story_id);
		
		if ($this->isPost())
		{				
			if($story->user_id !== $visitor->user_id){
				return $this->error(\XF::phrase('xtr_story_you_do_not_have_perm_manage_story'));
			}
			
			$story->story_create_date = \XF::$time;
			$story->save();
			
			return $this->redirect($this->buildLink('account/stories'), '', true);
		}
		else{
			if($story->user_id !== $visitor->user_id){
				return $this->error(\XF::phrase('xtr_story_you_do_not_have_perm_manage_story'));
			}
			
			$viewParams = [
				'story' => $story
			];
			
			return $this->view('XENTR\Story:Delete', 'xentr_member_story_recover', $viewParams);
		}
	}

    public static function getActivityDetails(array $activities){
        return 'Viewing stories';
    }

    /**
     * @param string $id
     * @param array|string|null $with
     * @param null|string $phraseKey
     *
     * @return \XENTR\Stories\Entity\Story
     */
    protected function assertStoryExists($id, $with = null, $phraseKey = null){
        return $this->assertRecordExists('XENTR\Stories:Story', $id, $with, $phraseKey);
    }

	/**
	 * @param int $userId
	 * @param array $extraWith
	 * @param bool $basicProfileOnly
	 *
	 * @return \XF\Entity\User
	 *
	 * @throws \XF\Mvc\Reply\Exception
	 */
	protected function assertViewableUser($userId, array $extraWith = [], $basicProfileOnly = false)
	{
		$extraWith[] = 'Option';
		$extraWith[] = 'Privacy';
		$extraWith[] = 'Profile';
		array_unique($extraWith);

		/** @var \XF\Entity\User $user */
		$user = $this->em()->find('XF:User', $userId, $extraWith);
		if (!$user)
		{
			throw $this->exception($this->notFound(\XF::phrase('requested_user_not_found')));
		}

		$canView = $basicProfileOnly ? $user->canViewBasicProfile($error) : $user->canViewFullProfile($error);
		if (!$canView)
		{
			throw $this->exception($this->noPermission($error));
		}

		return $user;
	}

}