<?php

/*
 * Created on 01.02.2021
 * HomePage: https://xentr.net
 * Copyright (c) 2019-2021 XENTR | XenForo Add-ons - Styles -  All Rights Reserved
 */

namespace XENTR\Stories\Service;


use XF\Service\AbstractService;
use XF\Util\File;
use XENTR\Stories\Entity\Story;

class StoryAttachment extends AbstractService
{
    /**
     * @var \XENTR\Stories\Entity\Story
     */
	 
    protected $story;

    protected $fileName;

    protected $width;

    protected $height;

    protected $type;

    public $extension;

    protected $error = null;

    public function __construct(\XF\App $app, Story $story)
    {
        parent::__construct($app);
        $this->story = $story;
    }

    public function getStory()
    {
        return $this->story;
    }

    public function getError()
    {
        return $this->error;
    }

    public function setStory($fileName){
        if (!$this->validateStory($fileName, $error))
        {
            $this->error = $error;
            $this->fileName = null;
            return false;
        }

        $this->fileName = $fileName;
        return true;
    }

    public function setStoryFromUpload(\XF\Http\Upload $upload){
        $constraints = $this->getStoryDefaultAttachmentConstraints();
        $constraints = $this->applyStoryVideoAttachmentConstraints($constraints);

        $upload->applyConstraints($constraints);

        if(!$upload->isValid($errors)){
            $this->error = reset($errors);
            return false;
        }

        $this->deleteStoryFlies();
        $this->extension = $upload->getExtension();

        return $this->setStory($upload->getTempFile());
    }

    public function validateStory($fileName, &$error = null)
    {
        $error = null;

        if (!file_exists($fileName))
        {
            $error = \XF::phrase('provided_file_is_not_valid_image');
            return false;
        }
        
		if (!is_readable($fileName))
        {
            $error = \XF::phrase('xtr_story_provided_file_is_not_readable_image');
            return false;
        }
		
		// FileWrapper class handles validating that the file is readable, image type is allowed, etc

        return true;
    }

    public function updateStory(){
        if (!$this->fileName) {
            throw new \LogicException("No source file for story set");
        }

        $outputFile = $this->fileName;

        if (!$outputFile) {
            throw new \RuntimeException("Failed to save story_attachment to temporary file; check internal_data/data permissions");
        }

        $dataFile = $this->getAbstractetStoryPath($this->extension);
        File::copyFileToAbstractedPath($outputFile, $dataFile);

        $this->getStory()->set('story_attachment_extension', $this->extension);
        $this->getStory()->save();

        return true;
    }

    public function deleteStory(){
        $this->deleteStoryFlies();

        return true;
    }

    public function deleteStoryFlies(){
        File::deleteFromAbstractedPath($this->getAbstractetStoryPath($this->getStory()->story_attachment_extension));
    }

    public function getAbstractetStoryPath($extension)
    {
        $story_id = $this->getStory()->story_id;
        //$story_create_date = $this->getStory()->story_create_date;

        return sprintf('data://stories/%d.%s',
            $story_id,
            $extension
        );
    }

    public function getStoryDefaultAttachmentConstraints()
    {
        $options = \XF::options();

        return [
            'extensions' => array_keys(\XF::app()->inlineImageTypes),
            'size' => $options->xtrStoryAttachtMaxFileSize * 1024,
            'width' => $options->xtrStoryAttachtMaxDimensions['width'],
            'height' => $options->xtrStoryAttachtMaxDimensions['height'],
            'count' => 1
        ];
    }

    public function applyStoryVideoAttachmentConstraints(array $constraints)
    {
        $options = \XF::options();

        /** @var \XF\Repository\Attachment $attachRepo */
        $attachRepo = \XF::repository('XF:Attachment');

        if (!empty($options->xtrStoryVideoMaxFileSize['enabled']))
        {
            $constraints['extensions'] = array_unique(array_merge(
                $constraints['extensions'],
                $attachRepo->getVideoAttachmentExtensions()
            ));

            $size = $options->xtrStoryVideoMaxFileSize['size'];
            $constraints['video_size'] = $size * 1024;
        }

        return $constraints;
    }

}