<?php

/*
 * Created on 01.02.2021 - Updated 22.08.2021
 * HomePage: https://xentr.net
 * Copyright (c) 2019-2021 XENTR | XenForo Add-ons - Styles -  All Rights Reserved
 */

namespace XENTR\Stories;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

class Setup extends AbstractSetup
{
	
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;
	
    public function installStep1()
    {
        $this->schemaManager()->createTable('xf_xtr_story', function(Create $table)
        {
            $table->addColumn('story_id', 'int')->autoIncrement(true, true);				//Story ID.
            $table->addColumn('story_message', 'varchar', 255)->setDefault('');				//Story title message.
            $table->addColumn('story_sticky', 'bool')->setDefault(0);						//Story sticky.
            $table->addColumn('story_message_url', 'varchar', 255)->setDefault('');			//Story title url.
			$table->addColumn('story_external_media', 'varchar', 255)->setDefault('');		//Story external media (video or image) URL.
            $table->addColumn('user_id', 'int');											//Story owner.
            $table->addColumn('story_deleted', 'bool')->setDefault(0);						//Story deleted action.
            $table->addColumn('story_create_date', 'int');									//Story create date.
            $table->addColumn('story_attachment_extension', 'varchar', 10)->setDefault('');	//Story attachment extensions.
			$table->addKey('story_create_date');											//Story addKey.
        });
    }

    public function installStep2()
    {
        $this->schemaManager()->alterTable('xf_user_privacy', function(Alter $table)
        {
			$table->addColumn('allow_story_view', 'enum')->values(['everyone','members','followed','none'])->setDefault('everyone');
        });
    }
	
	/** Upgrade step for version 1.0.1 - Story messages */
    public function upgrade1000170Step1()
	{
		$this->schemaManager()->alterTable('xf_xtr_story', function(Alter $table)
        {
            $table->addColumn('story_message_url', 'varchar', 255)->setDefault('');
			$table->addColumn('story_sticky', 'bool')->setDefault(0);	
        });
	}
	
	/** Upgrade step for version 1.0.8 - Set user privacy option to default everyone */
	public function upgrade1000870Step1(){
	
		$db = \XF::db();
		$db->query("
			UPDATE xf_user_privacy SET allow_story_view = 'everyone';
		");
	}

    public function uninstallStep1(){
        $sm = $this->schemaManager();

        $sm->dropTable(
            'xf_xtr_story'
        );
    }
	
    public function uninstallStep2(){
        $sm = $this->schemaManager();

        $sm->alterTable('xf_user_privacy', function(Alter $table)
        {
            $table->dropColumns(['allow_story_view']);
        });
    }
	
}